#ifndef RTPMANAGER_HPP
#define RTPMANAGER_HPP

/**
 * @file   RtpManager.hpp
 * @author Dan Weber
 * @date   Sun Mar  6 21:31:55 2005
 * 
 * @brief  Rtp Management Class
 * 
 * 
 */


#include <string>
#include <queue>
#include <list>
#include <map>
#include "RtpSession.hpp"
#include "RtpWrapper.hpp"


typedef unsigned short ushort;

/// Structure to store information about given RtpWrappers
typedef struct {
    /// Underlying RtpWrapper
    RtpWrapper* sessionwrapper;
    /// Host Address of RtpWrapper
    InetHostAddress host;
    /// Rtp Port bound to
    int rtpport;
    /// Rtcp Port bound to
    int rtcpport;
} rtpinfo_t;



/// Rtp Management Class
class RtpManager {
private:
    /// Map binding session names to rtp information
    std::map<std::string,rtpinfo_t,stringcompare> sessionmap;
    /// Local address
    InetHostAddress localaddr;
public:
/** 
 * Default Constructor for RtpManager instance
 * 
 */
    RtpManager();
/** 
 * Creates an RtpWrapper session and adds it to the map 
 * 
 * @param xmname XM Session Name
 * @param xmpath Path to XM Fifo
 * @param port Port to bind to
 * 
 * @return True on Success
 */
    bool createSession(std::string& xmname, std::string& xmpath,
		    unsigned short port);
/** 
 * Removes an RtpWrapper from the Map
 * 
 * @param xmname XM Session Name
 * 
 * @return True on Success
 * @todo Needs to be tested 
 */
    bool removeSession(std::string& xmname);
/** 
 * Retrieves the port of a given session
 * 
 * @param xmname XM Session Name
 * 
 * @return Port Number
 */
    unsigned short getPort(std::string& xmname);
/** 
 * Adds a destination to an RtpSession
 * 
 * @param xmname XM Session Name
 * @param identifier Identifier of Destination
 * @param destip Destination IP
 * @param port Destination Port
 * 
 * @return True on Success
 */
    bool addDestination(std::string& xmname, std::string& identifier,
		    	std::string& destip,
		    	unsigned short port);
/** 
 * Deletes a destionation from a RtpSessoin
 * 
 * @param xmname XM Session Name
 * @param identifier Identifier of Destination
 * 
 * @return True on Success
 */
    bool delDestination(std::string& xmname, std::string& identifier);
    /// Default Destructor
    ~RtpManager();
};
#endif
