#include <iostream>
#include "RtpSession.hpp"
#include <sys/types.h>
#include "g711.h"

using namespace std;
static const int PERIOD = 20;
static const int SAMPLING_RATE = 8000;
static const int PACKET_SIZE = 160;

inline void RtpSession::translate() {
  for (int i = 0; i < PACKET_SIZE; ++i) {
    assert (-8192 < v16[i] < 8191);
    v8[i] = linear2ulaw(v16[i]);
  }
}


RtpSession::RtpSession(const InetHostAddress& ia, 
			const unsigned short port,
			char* xmfifopath) : SymmetricRTPSession (ia,port) {
	xmpath = xmfifopath;
	// Set Payload to PCMU/ULAW
	setPayloadFormat(StaticPayloadFormat(sptPCMU));
	setSchedulingTimeout(10000);
	// Add the destination of the null rtpsession
	assert(addDestination(ia,15000));
	// Open the audio stream fifo
	assert(openFile());
}

bool RtpSession::openFile() {
	audioinput = ::open(xmpath,O_RDONLY);
	if (audioinput == -1) {
		printf("Error: %s, XM Path: %s",strerror(errno),xmpath);
		return false;
	}
	else
		return true;
}

void RtpSession::writeRtp() {
	setCancel(cancelImmediate);
	int count = PACKET_SIZE;
	for( unsigned int i = 0; count > 0; i++) {
		delete (getData(getFirstTimestamp()));
		count = ::read(audioinput,v16,PACKET_SIZE*2);
		if (count > 0) {
		  translate();
		  // Send some read data
		  putData(i*PACKET_SIZE,v8,PACKET_SIZE);
		  // putData(getCurrentTimestamp(),v8,PACKET_SIZE);
		}
	}
}

void RtpSession::onGotSR(SyncSource& source, SendReport& SR, uint8 blocks) {
  SymmetricRTPSession::onGotSR(source,SR,blocks);
  cout << "got an sr report from "
       << source.getNetworkAddress() << ":" 
       << source.getControlTransportPort()
       << endl;
}

RtpSession::~RtpSession() {}
