
#ifndef RtpHandler_HPP
#define RtpHandler_HPP
#include <cstdlib>
#include <cassert>
#include <sys/types.h>
#include <ccrtp/rtp.h>
// #include "SipHandler.hpp"
#ifdef  CCXX_NAMESPACES
using namespace ost;
using namespace std;
#endif

/**
 * @file   RtpSession.hpp
 * @author Dan Weber
 * @date   Sun Mar  6 21:20:44 2005
 * 
 * @brief  RtpSession class for handling individual audio sessions
 * 
 * 
 */

/// Rtp Session Class
class RtpSession : public SymmetricRTPSession, public TimerPort {
private:
	struct timeval *last_read;
	bool sent_rtp;
	char* xmpath;
	u_int8_t v8[160];
	int16_t v16[160];
	int audioinput; // file descriptor
public:
  /** 
   * Constructs an RtpSession instance
   * 
   * @param ia Local IP Address
   * @param port Local Port
   * @param xmfifopath Path to XM Ulaw Fifo Buffer
   * 
   */
  RtpSession(const InetHostAddress& ia, const unsigned short port, char* xmfifopath);

  /** 
   * 
   * Default Destructor
   * 
   */
  ~RtpSession();

  /** 
   * Handler function called upon receival of an Sender Report
   * 
   * @param source Synchronization Source
   * @param SR Sender Report
   * @param blocks Sender Report blocks
   */
  void onGotSR(SyncSource& source, SendReport& SR, uint8 blocks);

  /** 
   * Opens the XM File fifo
   * 
   * 
   * @return True on Success
   */
  bool openFile();

  /** 
   * Writes the rtp to the destinations of the session
   * 
   */
  void writeRtp();

  void translate();


};
#endif
