#include "RtpWrapper.hpp"

RtpWrapper::RtpWrapper(const InetHostAddress& ia,
	  	       const unsigned short port,
		       const std::string& xmfifopath) {
	session = new RtpSession(ia,port,const_cast<char*>(xmfifopath.c_str()));
}

void RtpWrapper::run(void) {
	session->startRunning();
	session->writeRtp();
}

bool RtpWrapper::addDestination(std::string& identifier,
				std::string& dip,
				unsigned short port) {
	if (dests.find(identifier) == dests.end()) {
		rtpdest_t dest;
		InetHostAddress ipaddr(dip.c_str());
		assert(session->addDestination(ipaddr,port));
		std::cerr << "Destination added: " << ipaddr.getHostname() << std::endl;
		dest.destip = ipaddr;
		dest.port = port;
		dests[identifier] = dest;
	}
	return true;
}

bool RtpWrapper::delDestination(std::string& identifier) {
	if(dests.find(identifier) != dests.end()) {
		std::cerr << "Deleting Destination for identifer: " << identifier << std::endl;
		rtpdest_t& dest = dests[identifier];
		assert(session->forgetDestination(dest.destip,dest.port));
		dests.erase(identifier);
	}
	return true;
}

RtpWrapper::~RtpWrapper() {
	delete session;
}
