#ifndef RTPWRAPPER_HPP
#define RTPWRAPPER_HPP
#include "RtpSession.hpp"
#include <map>
#include <string>

/**
 * @file   RtpWrapper.hpp
 * @author Dan Weber
 * @date   Sun Mar  6 21:22:15 2005
 * 
 * @brief  Wrappers around RtpSession
 * 
 * 
 */

/// Structure with a function for comparison of strings
struct stringcompare {
  /** 
   * Compares a string for use with maps
   * 
   * 
   * @return true if smaller
   */
	bool operator()(std::string a, std::string b) {
		return a < b;
    	}
};

/// Stores relevant information about rtp destinations
typedef struct {
	/// Destination IP 
	InetHostAddress destip;
	/// Destination Port
	unsigned short port;
} rtpdest_t;

/// Wrapper around RtpSession
class RtpWrapper : public Thread {
public:		
	/** 
	 * Constructs an RtpWrapper. This should be 
	 * used in place of calling RtpSession directly
	 * 
	 * @param ia Local Inet Address 
	 * @param port Local Port
	 * @param xmfifopath Path to XM Fifo buffer
	 * 
	 */
	RtpWrapper(const InetHostAddress& ia,
		   const unsigned short port,
		   const std::string& xmfifopath);
	/// Default Destructor
	~RtpWrapper();
	/** 
	 * Adds a Destination for Rtp to the RtpSession
	 * 
	 * @param identifier Identifier for the Destination
	 * @param dip Destination IP
	 * @param port Destination Port
	 * 
	 * @return True on Success 
	 */
	bool addDestination(std::string& identifier,
				std::string& dip,
			    unsigned short port);
	/** 
	 * Removes a Destination from an RtpSession
	 * 
	 * @param identifier Identifier for Destination
	 * 
	 * @return True on Success
	 */
	bool delDestination(std::string& identifier);
private:
	/// Map to store identifiers with destination information
	std::map<std::string,rtpdest_t,stringcompare> dests;
	/// Underlying RtpSession
	RtpSession* session;
	/// Thread Function 
	void run(void);
};
#endif
