#include "SipHandler.hpp"
#include <cstdlib>
#include <dotconf++/dotconfpp.h>
#include <sched.h>


typedef struct {
        std::string name;
        std::string path;
        int port;
} configentry_t;


bool parseAudioInput(const DOTCONFDocument* conf,std::list<configentry_t>& sets) {
	const DOTCONFDocumentNode *pnode = NULL;
	
	while((pnode = conf->findNode("AudioInput", NULL, pnode))){
		configentry_t confe;
		confe.name = pnode->getValue();
		if (confe.name.size() != 5) {
			printf("file %s, line %d: %s must be five characters\n",pnode->getConfigurationFileName(),
			       pnode->getConfigurationLineNumber(),pnode->getName());
			return false;
		}
		
		const DOTCONFDocumentNode * vnode;
		// Path
		assert(vnode = conf->findNode("Path", pnode));
		confe.path = vnode->getValue();
		
		// Port
		assert(vnode = conf->findNode("Port", pnode));
		confe.port = std::atoi(vnode->getValue());
		sets.push_back(confe);
	}
	return true;
}

using namespace resip;
int main (int argc, char* argv[]) {
	assert(argc > 1);
	struct sched_param schparam;
	schparam.sched_priority = 50;	
	sched_setscheduler(0,SCHED_FIFO,&schparam);

  	RtpManager* rtpman = new RtpManager;
  	SipHandler* sip = new SipHandler(rtpman,argv[1]);
	DOTCONFDocument* conf = new DOTCONFDocument(DOTCONFDocument::CASEINSENSETIVE);
	std::list<configentry_t> sets;

	if(conf->setContent("/etc/musiconhold.conf") == -1)
		return -1;
	if(!parseAudioInput(conf,sets))
		return -1;
	// Lets eliminate the parser from the memory leak check by deleting it after its use
	delete conf;
	for (std::list<configentry_t>::iterator i = sets.begin(); i != sets.end(); i++)
		rtpman->createSession(i->name,i->path,i->port);	
  	sip->run();
  	sip->join();
	

  
  return 0;
}

			
